#!/usr/bin/env python

from ..version import __name__

import os, sys, subprocess

# Is terminal ANSI/VT100 compatible
if os.getenv('TERM') in (
        'xterm',
        'vt100',
        'linux',
        'eterm-color',
        'screen',
    ):
    has_colors = True
else:
    try:
        # Eshell
        ppid = os.getppid()
        has_colors = (subprocess.getoutput('ps -p %d -ocomm=' % ppid)
                      == 'emacs')
    except:
        has_colors = False

# ANSI/VT100 escape code
# http://en.wikipedia.org/wiki/ANSI_escape_code
colors = {
    'none': '',
    'reset': '\033[0m',

    'black': '\033[30m',
    'bold-black': '\033[30;1m',
    'dark-gray': '\033[90m',
    'bold-dark-gray': '\033[90;1m',

    'red': '\033[31m',
    'bold-red': '\033[31;1m',
    'light-red': '\033[91m',
    'bold-light-red': '\033[91;1m',

    'green': '\033[32m',
    'bold-green': '\033[32;1m',
    'light-green': '\033[92m',
    'bold-light-green': '\033[92;1m',

    'yellow': '\033[33m',
    'bold-yellow': '\033[33;1m',
    'light-yellow': '\033[93m',
    'bold-light-yellow': '\033[93;1m',

    'blue': '\033[34m',
    'bold-blue': '\033[34;1m',
    'light-blue': '\033[94m',
    'bold-light-blue': '\033[94;1m',

    'magenta': '\033[35m',
    'bold-magenta': '\033[35;1m',
    'light-magenta': '\033[95m',
    'bold-light-magenta': '\033[95;1m',

    'cyan': '\033[36m',
    'bold-cyan': '\033[36;1m',
    'light-cyan': '\033[96m',
    'bold-light-cyan': '\033[96;1m',

    'light-gray': '\033[37m',
    'bold-light-gray': '\033[37;1m',
    'white': '\033[97m',
    'bold-white': '\033[97;1m',
}

def underlined(text):
    """Returns an underlined text.
    """
    return "\33[4m%s\33[24m" % text if has_colors else text

def println(text, color=None, ostream=sys.stdout):
    """Prints a text line to stream.
    """
    if has_colors and color in colors:
        ostream.write("{0}{1}{2}\n".format(colors[color], text, colors['reset']))
    else:
        ostream.write("{0}\n".format(text))

def printlog(message, color=None, ostream=sys.stderr):
    """Prints a log message to stream.
    """
    if has_colors and color in colors:
        ostream.write("{0}{1}: {2}{3}\n".format(colors[color], __name__, message, colors['reset']))
    else:
        ostream.write("{0}: {1}\n".format(__name__, message))

def i(message, ostream=sys.stderr):
    """Sends an info log message.
    """
    printlog(message,
             None,
             ostream=ostream)

def d(message, ostream=sys.stderr):
    """Sends a debug log message.
    """
    printlog(message,
             'blue' if has_colors else None,
             ostream=ostream)

def w(message, ostream=sys.stderr):
    """Sends a warning log message.
    """
    printlog(message,
             'yellow' if has_colors else None,
             ostream=ostream)

def e(message, ostream=sys.stderr):
    """Sends an error log message.
    """
    printlog(message,
             'bold-yellow' if has_colors else None,
             ostream=ostream)

def wtf(message, ostream=sys.stderr):
    """What a Terrible Failure.
    """
    printlog(message,
             'bold-red' if has_colors else None,
             ostream=ostream)



#!/usr/bin/env python

__all__ = ['vine_download']

from ..common import *

def vine_download(url, output_dir = '.', merge = True, info_only = False):
    html = get_html(url)
    
    title = r1(r'<meta property="og:title" content="([^"]*)"', html)
    url = "http:" + r1(r'<source src="([^"]*)"', html)
    type, ext, size = url_info(url)
    
    print_info(site_info, title, type, size)
    if not info_only:
        download_urls([url], title, ext, size, output_dir, merge = merge)

site_info = "Vine.co"
download = vine_download
download_playlist = playlist_not_supported('vine')



#!/usr/bin/env python

# Original code from:
# http://xiaoxia.org/2011/03/26/using-python-to-write-a-local-sogou-proxy-server-procedures/

from . import log

from http.client import HTTPResponse
from http.server import BaseHTTPRequestHandler, HTTPServer
from socketserver import ThreadingMixIn
from threading import Thread
import random, socket, struct, sys, time

def sogou_proxy_server(
    host=("0.0.0.0", 0),
    network_env='CERNET',
    ostream=sys.stderr):
    """Returns a Sogou proxy server object.
    """

    x_sogou_auth = '9CD285F1E7ADB0BD403C22AD1D545F40/30/853edc6d49ba4e27'
    proxy_host = 'h0.cnc.bj.ie.sogou.com'
    proxy_port = 80

    def sogou_hash(t, host):
        s = (t + host + 'SogouExplorerProxy').encode('ascii')
        code = len(s)
        dwords = int(len(s) / 4)
        rest = len(s) % 4
        v = struct.unpack(str(dwords) + 'i' + str(rest) + 's', s)
        for vv in v:
            if type(vv) != bytes:
                a = (vv & 0xFFFF)
                b = (vv >> 16)
                code += a
                code = code ^ (((code << 5) ^ b) << 0xb)
                # To avoid overflows
                code &= 0xffffffff
                code += code >> 0xb
        if rest == 3:
            code += s[len(s) - 2] * 256 + s[len(s) - 3]
            code = code ^ ((code ^ (s[len(s) - 1]) * 4) << 0x10)
            code &= 0xffffffff
            code += code >> 0xb
        elif rest == 2:
            code += (s[len(s) - 1]) * 256 + (s[len(s) - 2])
            code ^= code << 0xb
            code &= 0xffffffff
            code += code >> 0x11
        elif rest == 1:
            code += s[len(s) - 1]
            code ^= code << 0xa
            code &= 0xffffffff
            code += code >> 0x1
        code ^= code * 8
        code &= 0xffffffff
        code += code >> 5
        code ^= code << 4
        code = code & 0xffffffff
        code += code >> 0x11
        code ^= code << 0x19
        code = code & 0xffffffff
        code += code >> 6
        code = code & 0xffffffff
        return hex(code)[2:].rstrip('L').zfill(8)

    class Handler(BaseHTTPRequestHandler):
        _socket = None
        def do_proxy(self):
            try:
                if self._socket is None:
                    self._socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                    self._socket.connect((proxy_host, proxy_port))
                self._socket.send(self.requestline.encode('ascii') + b'\r\n')
                log.d(self.requestline, ostream)

                # Add Sogou Verification Tags
                self.headers['X-Sogou-Auth'] = x_sogou_auth
                t = hex(int(time.time()))[2:].rstrip('L').zfill(8)
                self.headers['X-Sogou-Tag'] = sogou_hash(t, self.headers['Host'])
                self.headers['X-Sogou-Timestamp'] = t
                self._socket.send(str(self.headers).encode('ascii') + b'\r\n')

                # Send POST data
                if self.command == 'POST':
                    self._socket.send(self.rfile.read(int(self.headers['Content-Length'])))
                response = HTTPResponse(self._socket, method=self.command)
                response.begin()

                # Response
                status = 'HTTP/1.1 %s %s' % (response.status, response.reason)
                self.wfile.write(status.encode('ascii') + b'\r\n')
                h = ''
                for hh, vv in response.getheaders():
                    if hh.upper() != 'TRANSFER-ENCODING':
                        h += hh + ': ' + vv + '\r\n'
                self.wfile.write(h.encode('ascii') + b'\r\n')
                while True:
                    response_data = response.read(8192)
                    if len(response_data) == 0:
                        break
                    self.wfile.write(response_data)

            except socket.error:
                log.e('Socket error for ' + self.requestline, ostream)

        def do_POST(self):
            self.do_proxy()

        def do_GET(self):
            self.do_proxy()

    class ThreadingHTTPServer(ThreadingMixIn, HTTPServer):
        pass

    # Server starts
    log.printlog('Sogou Proxy Mini-Server', color='bold-green', ostream=ostream)

    try:
        server = ThreadingHTTPServer(host, Handler)
    except Exception as ex:
        log.wtf("Socket error: %s" % ex, ostream)
        exit(1)
    host = server.server_address

    if network_env.upper() == 'CERNET':
        proxy_host = 'h%s.edu.bj.ie.sogou.com' % random.randint(0, 10)
    elif network_env.upper() == 'CTCNET':
        proxy_host = 'h%s.ctc.bj.ie.sogou.com' % random.randint(0, 3)
    elif network_env.upper() == 'CNCNET':
        proxy_host = 'h%s.cnc.bj.ie.sogou.com' % random.randint(0, 3)
    elif network_env.upper() == 'DXT':
        proxy_host = 'h%s.dxt.bj.ie.sogou.com' % random.randint(0, 10)
    else:
        proxy_host = 'h%s.edu.bj.ie.sogou.com' % random.randint(0, 10)

    log.i('Remote host: %s' % log.underlined(proxy_host), ostream)
    log.i('Proxy server running on %s' %
        log.underlined("%s:%s" % host), ostream)

    return server



#!/usr/bin/env python

__all__ = ['ted_download']

from ..common import *

def ted_download(url, output_dir = '.', merge = True, info_only = False):
    page = get_html(url).split("\n")
    for line in page:
        if line.find("<title>") > -1:
            title = line.replace("<title>", "").replace("</title>", "").replace("\t", "")
            title = title[:title.find(' | ')]
        if line.find("no-flash-video-download") > -1:
            url = line.replace('<a id="no-flash-video-download" href="', "").replace(" ", "").replace("\t", "").replace(".mp4", "-480p.mp4")
            url = url[:url.find('"')]
            type, ext, size = url_info(url)
            print_info(site_info, title, type, size)
            if not info_only:
                download_urls([url], title, ext, size, output_dir, merge=merge)
            break

site_info = "TED.com"
download = ted_download
download_playlist = playlist_not_supported('ted')



#!/usr/bin/env python

__all__ = ['acfun_download']

from ..common import *

from .qq import qq_download_by_id
from .sina import sina_download_by_vid
from .tudou import tudou_download_by_iid
from .youku import youku_download_by_id

import json, re

def get_srt_json(id):
    url = 'http://comment.acfun.tv/%s.json' % id
    return get_html(url)

def get_srt_lock_json(id):
    url = 'http://comment.acfun.tv/%s_lock.json' % id
    return get_html(url)

def acfun_download_by_id(id, title = None, output_dir = '.', merge = True, info_only = False):
    info = json.loads(get_html('http://wenzhou.acfun.tv/api/getVideoByID.aspx?vid=' + id))
    t = info['vtype']
    vid = info['vid']
    if t == 'sina':
        sina_download_by_vid(vid, title, output_dir = output_dir, merge = merge, info_only = info_only)
    elif t == 'youku':
        youku_download_by_id(vid, title, output_dir = output_dir, merge = merge, info_only = info_only)
    elif t == 'tudou':
        tudou_download_by_iid(vid, title, output_dir = output_dir, merge = merge, info_only = info_only)
    elif t == 'qq':
        qq_download_by_id(vid, title, output_dir = output_dir, merge = merge, info_only = info_only)
    else:
        raise NotImplementedError(t)
    
    if not info_only:
        try:
            print('Downloading %s ...' % (title + '.cmt.json'))
            cmt = get_srt_json(vid)
            with open(os.path.join(output_dir, title + '.cmt.json'), 'w') as x:
                x.write(cmt)
            print('Downloading %s ...' % (title + '.cmt_lock.json'))
            cmt = get_srt_lock_json(vid)
            with open(os.path.join(output_dir, title + '.cmt_lock.json'), 'w') as x:
                x.write(cmt)
        except:
            pass

def acfun_download(url, output_dir = '.', merge = True, info_only = False):
    assert re.match(r'http://[^\.]+.acfun.tv/v/ac(\d+)', url)
    html = get_html(url)
    
    title = r1(r'<h1 id="title-article" class="title"[^<>]*>([^<>]+)<', html)
    assert title
    title = unescape_html(title)
    title = escape_file_path(title)
    title = title.replace(' - AcFun.tv', '')
    
    id = r1(r"\[Video\](\d+)\[/Video\]", html) or r1(r"\[video\](\d+)\[/video\]", html)
    if not id:
        id = r1(r"src=\"/newflvplayer/player.*id=(\d+)", html)
        sina_download_by_vid(id, title, output_dir = output_dir, merge = merge, info_only = info_only)
    else:
        acfun_download_by_id(id, title, output_dir = output_dir, merge = merge, info_only = info_only)

site_info = "AcFun.tv"
download = acfun_download
download_playlist = playlist_not_supported('acfun')
